/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"

#include "initng_depend.h"

s_entry ALSO_START = { "also_start", STRINGS, 10 };
s_entry ALSO_STOP = { "also_stop", STRINGS, 9 };


/* returns TRUE if all use deps are started */
static int check_use(active_h * service, e_a_status status)
{
    active_h **deps = NULL;
    int i;
    
    assert(service);
    assert(service->name);
    

    if (status != START_DEP_MET) return(TRUE);
    
    deps=active_db_dep_on(service);
    for(i=0;deps[i];i++)
    {
        switch (deps[i]->a_status)
        {
            case RUNNING:
            case DONE:
                continue;                   /* this use is met */
            default:
                D_("\"%s\" still waits for use \"%s\" that has status \"%s\".\n", service->name, deps[i]->name, active_db_get_status_string(deps[i]->a_status));
		free(deps);
                return (FALSE);
        }
    }
    
    free(deps);
    return(TRUE);
}

int service_state(active_h * service)
{
    char *tmp = NULL;
    active_h *current;
    
    assert(service);
    assert(service->name);

    switch (service->a_status)
    {
        case LOADING:
    	    while ((tmp = active_db_get_next_string(&NEED, service, tmp)))
            {
		/* HACK WARNING
		 * service name starting with "virtual/" is no idea to start, sience
		 * they wont exists anyway, this is a waist of resources.
		 */
                if (strncmp(tmp, "virtual/", 8) == 0)
                    continue;

                if (!start_new_service_named(tmp))
                {
                    F_("%s need dep %s coud not start!\n", service->name, tmp);
                }
            }
	    tmp=NULL;
            while ((tmp =
                    active_db_get_next_string(&ALSO_START, service, tmp)))
	    {
		/* HACK WARNING
		 * service name starting with "virtual/" is no idea to start, sience
		 * they wont exists anyway, this is a waist of resources.
		 */
                if (strncmp(tmp, "virtual/", 8) == 0)
                    continue;

                if (!start_new_service_named(tmp))
                {
                    F_("%s also_start %s coud not start!\n", service->name, tmp);
                }
	    }
            break;
        case STOPPED:
            /* Handle ALSO_STOP */
            while ((tmp =
                    active_db_get_next_string(&ALSO_STOP, service, tmp)))
                if ((current = active_db_find_by_name(tmp)))
                {
                    D_("service %s also stops %s\n", service->name, tmp);
                    stop_service(current);
                }
            break;
        default:
            break;
    }
    return (TRUE);
}


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&ALSO_START);
    initng_sdt_add(&ALSO_STOP);
    initng_add_hook(CONFIRM_ASTATUS_CHANGE, 50, &check_use);
    initng_add_hook(ASTATUS_CHANGE, 50, &service_state);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&ALSO_START);
    initng_sdt_del(&ALSO_STOP);
    initng_del_hook(CONFIRM_ASTATUS_CHANGE, &check_use);
    initng_del_hook(ASTATUS_CHANGE, &service_state);
}
