/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"

#include "initng_last.h"

s_entry LAST = { "last", SET, 4 };


/* returns TRUE if all use deps are started */
static int check_last(active_h * service, e_a_status status)
{
    active_h *current=NULL;
    active_h **deps;
    assert(service);
    assert(service->name);
    
    /* if we are trying to set status START_DEP_MET */
    if (status != START_DEP_MET) return(TRUE);

    /* And LAST is set. */
    if(!active_db_is(&LAST, service)) return(TRUE);
    
    deps=active_db_dep_on_me(service);
    /*
     * walk the active db, check if there is any other
     * service that are still starting and not also
     * have the last flag set.
     */
    while_active_db(current)
    {
	/* not check ourself */
	if(current==service) continue;
	    
		
	switch(current->a_status)
	{
	    case LOADING:
	    case STARTING:
	    case WAITING_FOR_START_DEP:
	    case START_LAUNCHED:
	    case START_DEP_MET:
		break;
	    default:
		continue; /* while_active_db */
	}
	
	
	/* if this service also shud be started last, continue */
	if(active_db_is(&LAST, current)) {
	    continue;
	}
	
	/* check so this service, are not waiting for our service that
	   shud be last, to be started */
	if(deps)
	{
	    int i=0;
	    while(deps[i] && deps[i]!=current)
		i++;
	    if(deps[i]==current) continue;
	}

	/*
	 * There is still services
	 * starting, wont allow
	 * a_status change!
	 */
	free(deps);
	return(FALSE);
	
    }

    /* accept this change */
    free(deps);
    return (TRUE);
}


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&LAST);
    initng_add_hook(CONFIRM_ASTATUS_CHANGE, 50, &check_last);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&LAST);
    initng_del_hook(CONFIRM_ASTATUS_CHANGE, &check_last);
}
