
/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"

#include "initng_reload.h"

#define SAVE_FILE "/tmp/active_db_save.v7"

s_command FAST_RELOAD = { 'c', "hot_reload", TRUE_OR_FALSE_COMMAND, NO_OPT,
    {(void *) &cmd_fast_reload},
    "Fast Reload"
};



static void read_adb_file(void)
{
    FILE *fil;
    active_h *new_entry = NULL;
    data_save_struct entry;
    s_data *d;
    int i;

    fil = fopen(SAVE_FILE, "r");

    if (!fil)
        return;

    while (!feof(fil))
    {
        if (!fread(&entry, sizeof(entry), 1, fil))
            continue;

        if (active_db_find_by_name(entry.name))
        {
            W_("Entry exists, wont create it!\n");
            continue;
        }

        if (!(new_entry = active_db_new(i_strdup(entry.name))))
        {
            F_("Cant create new active!\n");
            continue;
        }
        new_entry->a_status = entry.a_status;
	new_entry->type = entry.type;
        new_entry->time_got_status = entry.time_got_status;

        if (entry.start_p_pid)
        {
            new_entry->start_process = process_db_new(T_START);
            if (new_entry->start_process)
            {
                new_entry->start_process->pid = entry.start_p_pid;
                new_entry->start_process->stdout[0] = entry.start_stdout1;
                new_entry->start_process->stdout[1] = entry.start_stdout2;
            }
        }

        if (entry.stop_p_pid)
        {
            new_entry->stop_process = process_db_new(T_STOP);
            if (new_entry->stop_process)
            {
                new_entry->stop_process->pid = entry.stop_p_pid;
                new_entry->stop_process->stdout[0] = entry.stop_stdout1;
                new_entry->stop_process->stdout[1] = entry.stop_stdout2;
            }
        }
	
	i=0;
	while(entry.data[i].opt_type)
	{
	    printf("no %i\n", i);
	    d=(s_data *)initng_calloc(1, sizeof(s_data));
	    d->type=initng_sdt_find(entry.data[i].type);
	    if(!d->type)
	    {
		F_("Did not found %s!\n", entry.data[i].type);
		free(d);
		i++;
		continue;
	    }
	    switch(d->type->opt_type)
	    {
		case STRING:
		case STRINGS:
		    d->t.s=i_strdup(entry.data[i].t.s);
		    break;
		case INT:
		    d->t.i=entry.data[i].t.i;
		    break;
		default:
		    break;
	    }
	    list_add(&d->list, &new_entry->data.list);
	    i++;    
	}
        if (active_db_add(new_entry) != TRUE)
        {
            F_("Coud not add entry!\n");
            active_db_free(new_entry);
            continue;
        }
        W_("Added from " SAVE_FILE ": %s\n", entry.name);
    }

    fclose(fil);
    if (unlink(SAVE_FILE) != 0)
    {
        W_("Failed removing file " SAVE_FILE " !!!\n");
        return;
    }
}

static void cmd_fast_reload(char *arg, FILE * fd)
{
    FILE *fil;
    active_h *current, *q = NULL;
    data_save_struct entry;
    int i;
    s_data *c_d = NULL;

    (void) arg;

    fil = fopen(SAVE_FILE, "w+");

    /* walk the active_db */
    while_active_db_safe(current, q)
    {
        if (strlen(current->name) >= MAX_SERVICE_NAME_STRING_LEN)
        {
            F_("Service name is to long, it wont fit the fileformat spec! max is %i, wont save this service!\n", MAX_SERVICE_NAME_STRING_LEN);
            continue;
        }
        strcpy(entry.name, current->name);
        entry.a_status = current->a_status;
	entry.type = current->type;
        entry.time_got_status = current->time_got_status;

        /* set start process */
        if (current->start_process)
        {
            entry.start_p_pid = current->start_process->pid;
            entry.start_stdout1 = current->start_process->stdout[0];
            entry.start_stdout2 = current->start_process->stdout[1];
        }
        else
        {
            entry.start_p_pid = 0;
            entry.start_stdout1 = 1;
            entry.start_stdout2 = 2;
        }

        /* set stop process */
        if (current->stop_process)
        {
            entry.stop_p_pid = current->stop_process->pid;
            entry.stop_stdout1 = current->stop_process->stdout[0];
            entry.stop_stdout2 = current->stop_process->stdout[1];
        }
        else
        {
            entry.stop_p_pid = 0;
            entry.stop_stdout1 = 1;
            entry.stop_stdout2 = 2;
        }
	
	/* reset data */
	for(i=0;i<MAX_ENTRYS_FOR_SERVICE;i++)
	{
	    entry.data[i].opt_type=0;
	    entry.data[i].type[0]='\0';
	    entry.data[i].t.i=0;
	}
	
	i=0;
	list_for_each_entry(c_d, &current->data.list, list)
	{
	    if(!c_d->type) continue;
	    if(!c_d->type->opt_name) continue;
	    
	    entry.data[i].opt_type = c_d->type->opt_type;
	    strncpy(entry.data[i].type, c_d->type->opt_name, MAX_TYPE_STRING_LEN);
	    
	    /* copy the data */
	    switch(c_d->type->opt_type)
	    {
		case STRING:
		case STRINGS:
		    strncpy(entry.data[i].t.s, c_d->t.s, MAX_DATA_STRING_LEN);
		    break;
		case INT:
		    entry.data[i].t.i = c_d->t.i;
		    break;
		default:
		    break;
	    }
	    
	    /* save in next data entry row */
	    i++;
	    
	    /* maximum data entrys that can be saved */
	    if(i==MAX_ENTRYS_FOR_SERVICE) {
		F_("Maximum 20 data entrys / service can be saved!\n");
		break;
	    }
	}

        printf("Saving : %s\n", entry.name);
        fwrite(&entry, sizeof(entry), 1, fil);
    }

    fclose(fil);

    if(!g.i_am_init) _exit(0);
    
    {

        char *new_argv[] =
            { i_strdup("/sbin/initng"), i_strdup("--hot_reload"), NULL };
        execve("/sbin/initng", new_argv, environ);
    }
    F_("Failed to reload initng!\n");
}

int module_init(const char *version)
{
    D_("module_init(reload);\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    read_adb_file();
    initng_command_add(&FAST_RELOAD);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_command_del(&FAST_RELOAD);
}
