/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"

#include "initng_respawn.h"

s_entry RESPAWN = { "respawn", SET, 7 };

void initng_sdt_add_defaults(void)
{
    initng_sdt_add(&RESPAWN);
}


static int check_respawn(active_h *service, e_a_status status)
{

    assert (service);
    
    /* look for when initng are trying to set a servuce to STOPPED */
    if(status != STOPPED) return(TRUE);
    
    /* check that current status is RUNNING, so it realy died, and not is manually stopped */
    if(service->a_status != RUNNING) return(TRUE);
    
    /* make sure the initng is not exiting */
    if (g.sys_state == STATE_STOPPING) return(TRUE);
    
    /* check if the service have respawn enabled */
    if (!active_db_is(&RESPAWN, service)) return(TRUE);
    
    /* then respawn it */
    service->a_status=LOADING; /* so that start_service will restart it */
    start_service(service);
    
    /* dont allow change top STOPPED, it is set to STARTING with start_service abow */
    return(FALSE);
}    


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&RESPAWN);
    initng_add_hook(CONFIRM_ASTATUS_CHANGE, 50, &check_respawn);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&RESPAWN);
    initng_del_hook(CONFIRM_ASTATUS_CHANGE, &check_respawn);
}
