
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"
#include "initng_global.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "initng_struct_data.h"
#include "initng_static_data_id.h"
#include "initng_toolbox.h"

#undef D_
#define D_(fmt, ...)

char *d_get_string(s_entry * type, struct list_head *address)
{
    s_data *current = NULL;

    assert(address);

    /* check so that data entry exists */
    if (!type)
    {
        F_("Type cant be zero!\n");
        return (NULL);
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;
	
    if (type->opt_type != STRING && type->opt_type != SCRIPT)
    {
        F_("Cand only fetch a type, that is a STRING || SCRIPT!\n");
        return (NULL);
    }

    /* walk thru all entrys, on adress */
    list_for_each_entry(current, address, list)
    {
        /* if this type is right, return string */
        if (current->type == type && current->t.s)
        {
            D_("got %s\n", current->t.s);
            return (current->t.s);
        }
    }

    /* did not get any match */
    D_("String type %s doesn't exist.\n", type->opt_name);
    return (NULL);
}

int d_get_int(s_entry * type, struct list_head *d)
{
    s_data *current = NULL;

    assert(d);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return (0);
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    if (type->opt_type != INT)
    {
        F_("Cand only fetch a type, that is a INT!\n");
        return (0);
    }

    /* walk thru all entrys on adress */
    list_for_each_entry(current, d, list)
    {
        /* if this type is right, return int */
        if (current->type == type && current->t.i)
            return (current->t.i);

    }

    /* did not get any match */
    D_("Int value, type %s doesn't exist in db.\n", type->opt_name);
    return (0);
}


char *d_get_next_string(s_entry * type, struct list_head *d, char *last)
{
    s_data *current = NULL;

    assert(d);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return (NULL);
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    if (type->opt_type != STRINGS)
    {
        F_("Cand only fetch a type, that is a STRINGS!\n");
        return (NULL);
    }

    /* walk thru all in d, will be but into current */
    list_for_each_entry_prev(current, d, list)
    {
        /* if the returned type is not that type we are looking for, continue */
        if (current->type != type)
            continue;

        if (current->t.s == NULL)
            return (NULL);
        /* if this is the one, we start looking after, reset last, and continue */
        if (last)
        {
            if (current->t.s == last)
            {
                last = NULL;
            }
            continue;
        }

        /* if this is last, return next string */
        return (current->t.s);
    }
    /* no match */
    return (NULL);
}

/* this has to be only one of */
void d_set_string(s_entry * type, struct list_head *d, char *string)
{
    s_data *current = NULL;

    assert(d);
    assert(string);


    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    if (type->opt_type != STRING && type->opt_type != SCRIPT)
    {
        F_(" \"%s\" is not an STRING || SCRIPT type, sleeping 1 sek ..", type->opt_name);
        sleep(1);
        return;
    }

    /* check the db, for an current entry to overwrite */
    list_for_each_entry(current, d, list)
    {
        if (current->type == type)
        {
            if (current->t.s)
                free(current->t.s);
            current->t.s = string;
            return;
        }
    }

    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->t.s = string;

    /* add this one */
    list_add(&(current->list), d);
}



/* this is a function to add a string, that it can exist many of */
void d_set_another_string(s_entry * type, struct list_head *d, char *string)
{
    s_data *current = NULL;

    assert(d);
    assert(string);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    if (type->opt_type != STRINGS)
    {
        F_(" \"%s\" is not an strings type!\n", type->opt_name);
        return;
    }

    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->t.s = string;

    /* add this one */
    list_add(&(current->list), d);
}

/* this has to be only one of */
void d_set_int(s_entry * type, struct list_head *d, int value)
{
    s_data *current = NULL;

    assert(d);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    if (type->opt_type != INT)
    {
        F_(" \"%s\" is not an int type!\n", type->opt_name);
        return;
    }

    /* check the db, for an current entry to overwrite */
    list_for_each_entry(current, d, list)
    {
        if (current->type == type)
        {
            current->t.i = value;
            return;
        }
    }
    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->t.i = value;

    /* add this one */
    list_add(&(current->list), d);
}



/* Walk through and count */
int d_count_type(s_entry * type, struct list_head *d)
{
    s_data *current = NULL;
    int count = 0;

    /* walk thru all entrys on adress */
    list_for_each_entry(current, d, list)
    {
        if (!type || current->type == type)
            count++;
    }

    return (count);
}

/* 
 * d_is can be used to check any type, if its in the db,
 * or just to check unset and set types.
 */
int d_is(s_entry * type, struct list_head *d)
{
    s_data *current = NULL;

    assert(d);
    if (!type)
    {
        F_("d_is with no type\n");
        return (FALSE);
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    /* walk thru all entrys on adress */
    list_for_each_entry(current, d, list)
    {
        if (current->type == type)
        {
            /* if it is a set type, meens it true when found in db */
            /* if it is a unset type, meens it true when not found in db */
            if (current->type->opt_type == UNSET)
                return (FALSE);
            else
                return (TRUE);
        }
    }

    if (type->opt_type == UNSET)
        return (TRUE);
    return (FALSE);
}

void d_set(s_entry * type, struct list_head *d)
{
    s_data *current = NULL;

    assert(d);
    if (!type)
    {
        F_("d_is with no type!\n");
        return;
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    if (type->opt_type != SET && type->opt_type != UNSET)
    {
        F_("It has to be an SET or UNSET type to d_set!\n");
        return;
    }

    /* check if its set alredy */
    if (type->opt_type == SET)
        if (d_is(type, d))
            return;

    /* check if its unset alderdy */
    if (type->opt_type == UNSET)
        if (!d_is(type, d))
            return;

    /* allocate the entry */
    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;

    /* add this one */
    list_add(&(current->list), d);
}

void d_remove(s_entry * type, struct list_head *d)
{
    struct list_head *pos, *q = NULL;
    s_data *current = NULL;

    assert(d);
    assert(type);

    if (!type)
    {
        F_("type not set!\n");
        return;
    }

    /* this might be an alias */
    while (type->opt_type == ALIAS && type->alias)
	type=type->alias;


    /* walk thru all entrys on adress */
    list_for_each_safe(pos, q, d)
    {
        current = list_entry(pos, s_data, list);
        if (current->type == type)
        {
            if (current->t.s &&
                (current->type->opt_type == STRING
                 || current->type->opt_type == STRINGS
                 || current->type->opt_type == SCRIPT))
            {
                D_("Freeing string \"%s\"\n", current->t.s);
                free(current->t.s);
            }
            list_del(pos);
            free(current);
        }
    }
}

/* walk through and kill all strings */
void d_remove_all(struct list_head *d)
{
    struct list_head *pos, *q = NULL;
    s_data *current = NULL;

    assert(d);

    /* walk thru all entrys on adress */
    list_for_each_safe(pos, q, d)
    {
        current = list_entry(pos, s_data, list);
        if (current->t.s
            && (current->type->opt_type == STRING
                || current->type->opt_type == STRINGS
                || current->type->opt_type == SCRIPT))
        {
            D_("Freeing string \"%s\"\n", current->t.s);
            free(current->t.s);
        }
        list_del(pos);
        free(current);
    }
}

/*
 * Walk the db, copy all strings in list from to to.
 */
void d_copy_all(struct list_head *from, struct list_head *to)
{
    s_data *tmp = NULL;
    s_data *current = NULL;

    list_for_each_entry(current, from, list)
    {
        tmp = (s_data *) initng_calloc(1, sizeof(s_data));
        memcpy(tmp, current, sizeof(s_data));

        /* copy the data */
        if (current->type)
            switch (current->type->opt_type)
            {
                case STRING:
                case STRINGS:
                case SCRIPT:
                    if (current->t.s)
                        tmp->t.s = i_strdup(current->t.s);
                    break;
                default:
                    break;
            }

        list_add(&tmp->list, to);
    }
}


