
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <CUnit/CUnit.h>
#include <CUnit/Basic.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#include "test_initng.h"
#include "../src/initng_struct_data.h"

static s_entry INT_ENTRY = { "int_entry", INT, 9 };
static s_entry STRING_ENTRY = { "string_entry", STRING, 12 };
static s_entry STRINGS_ENTRY = { "strings_entry", STRINGS, 13 };
static s_entry SET_ENTRY = { "set_entry", SET, 9 };
static s_entry UNSET_ENTRY = { "unset_entry", UNSET, 11 };
static s_entry SCRIPT_ENTRY = { "script_entry", SCRIPT, 12 };

static s_data local_data;


/* test d_get_string */
static void test_d_get_string(void)
{
    NEW_TEST();

    INIT_LIST_HEAD(&local_data.list);

    TEST("get an string, when no is set ... ");
    CU_ASSERT(d_get_string(&STRING_ENTRY, &local_data.list) == NULL)
        /* reset */
        d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set a strign, and get it ... ");
    /* set a string */
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_data_1"));
    /* make sure what we got, is what we set */
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &local_data.list),
                           "test_data_1");
    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);


    TEST("set a string and script, and get it ... ");
    /* set a string */
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_data_1"));
    d_set_string(&SCRIPT_ENTRY, &local_data.list, strdup("test_data_2"));
    /* make sure what we got, is what we set */
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &local_data.list),
                           "test_data_1");
    CU_ASSERT_STRING_EQUAL(d_get_string(&SCRIPT_ENTRY, &local_data.list),
                           "test_data_2");
    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set an int type, and try try to fetch a string ...");
    /* set an int */
    d_set_int(&INT_ENTRY, &local_data.list, 10);
    /* make sure we did not get something */
    CU_ASSERT(d_get_string(&INT_ENTRY, &local_data.list) == NULL);

    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_get_int(void)
{
    NEW_TEST();
    /* reset */
    INIT_LIST_HEAD(&local_data.list);

    TEST("Test 1, get an value not set.");
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 0);

    TEST("Test 2, set and get the integer ...");
    /* set an int */
    d_set_int(&INT_ENTRY, &local_data.list, 22);
    /* get it again. */
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 22);

    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("Test 3, set an string, and fetch it as an integer ...");
    /* set a string */
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_data_1"));
    /* get it again, make sure its ZERO */
    CU_ASSERT(d_get_int(&STRING_ENTRY, &local_data.list) == 0);

    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

}

static void test_d_get_next_string(void)
{
    char *string = NULL;

    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);

    TEST("get an value not set.");
    CU_ASSERT(d_get_next_string(&STRINGS_ENTRY, &local_data.list, string) ==
              0);

    TEST("set 2 strings");
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("FirstEntry"));
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("SecondEntry"));

    TEST("make sure there are 2 strings");
    /* make sure there is 2 strings */
    CU_ASSERT(d_count_type(&STRINGS_ENTRY, &local_data.list) == 2);

    TEST("make sure we got them both");
    string = NULL;
    while ((string =
            d_get_next_string(&STRINGS_ENTRY, &local_data.list, string)))
    {
        CU_ASSERT(string != NULL);
        CU_ASSERT(strcmp(string, "FirstEntry") == 0
                  || strcmp(string, "SecondEntry") == 0);
    }

    TEST("make sure we got them in order ..");
    string = NULL;
    string = d_get_next_string(&STRINGS_ENTRY, &local_data.list, string);
    CU_ASSERT(strcmp(string, "FirstEntry") == 0);
    string = d_get_next_string(&STRINGS_ENTRY, &local_data.list, string);
    CU_ASSERT(strcmp(string, "SecondEntry") == 0);

    TEST("also set an string, and try to fetch is as strings..");
    /* set a string */
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_data_1"));
    CU_ASSERT(d_get_next_string(&STRING_ENTRY, &local_data.list, NULL) ==
              NULL);

    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_set_another_string(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);

    TEST("set a string, and fetch it again.");
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("FirstEntry"));
    CU_ASSERT(strcmp
              (d_get_next_string(&STRINGS_ENTRY, &local_data.list, NULL),
               "FirstEntry") == 0);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);


    TEST("set 2 strings, and fetch first again, and check.");
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("FirstEntry"));
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("SecondEntry"));
    CU_ASSERT(strcmp
              (d_get_next_string(&STRINGS_ENTRY, &local_data.list, NULL),
               "FirstEntry") == 0);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("try to set STRINGS, to entrys, of wrong type.");
    d_set_another_string(&STRING_ENTRY, &local_data.list,
                         strdup("Shud not work"));
    d_set_another_string(&INT_ENTRY, &local_data.list,
                         strdup("Shud not work"));
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set a string, with no type \n");
    d_set_another_string(NULL, &local_data.list, strdup("Shud not work"));
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);


    /* will make an assert, so dont test 
       TEST("Test 3, set an empty string! \n");
       d_set_another_string(&STRINGS_ENTRY, &local_data.list, NULL);
       CU_ASSERT(list_empty(&local_data.list));
       d_remove_all(&local_data.list);
       INIT_LIST_HEAD(&local_data.list); */
}

static void test_d_set_string(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);

    TEST("set and get an string.");
    /* set a string */
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_string"));
    /* get that we set */
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &local_data.list),
                           "test_string");
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);


    TEST("set an string, and overwrite it, make sure we got the last entry");
    /* set a string */
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_data_a"));
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_data_b"));
    /* make sure what we got, is what we set */
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &local_data.list),
                           "test_data_b");
    /* reset */
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);


    TEST("try to set STRING, to entrys, of wrong type.");
    d_set_string(&STRINGS_ENTRY, &local_data.list, strdup("Shud not work"));
    d_set_string(&INT_ENTRY, &local_data.list, strdup("Shud not work"));
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set a string, with no type \n");
    d_set_string(NULL, &local_data.list, strdup("Shud not work"));
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

}

static void test_d_set_int(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);

    TEST("set an int, and try to read it");
    d_set_int(&INT_ENTRY, &local_data.list, 99);
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 99);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set and overwrite an int, and try to read it");
    d_set_int(&INT_ENTRY, &local_data.list, 99);
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 99);
    d_set_int(&INT_ENTRY, &local_data.list, 66);
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 66);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set som faied entrys, and check so that its empty.");
    d_set_int(&STRING_ENTRY, &local_data.list, 99);
    d_set_int(&STRINGS_ENTRY, &local_data.list, 99);
    d_set_int(&SET_ENTRY, &local_data.list, 99);
    d_set_int(&UNSET_ENTRY, &local_data.list, 99);
    d_set_int(&SCRIPT_ENTRY, &local_data.list, 99);
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set with empty type.");
    d_set_int(NULL, &local_data.list, 99);
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_count_type(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);

    TEST("set a few types, and count it ... ");
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_string"));
    d_set_int(&INT_ENTRY, &local_data.list, 99);
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("FirstEntry"));
    d_set_another_string(&STRINGS_ENTRY, &local_data.list,
                         strdup("SecondEntry"));

    CU_ASSERT(d_count_type(&STRINGS_ENTRY, &local_data.list) == 2);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_is(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);
    TEST("set an int, set, and string, and start checking.");
    d_set_int(&INT_ENTRY, &local_data.list, 99);
    d_set(&SET_ENTRY, &local_data.list);
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test_string"));

    /* check the right types */
    CU_ASSERT(d_is(&SET_ENTRY, &local_data.list) == TRUE);
    CU_ASSERT(d_is(&UNSET_ENTRY, &local_data.list) == TRUE);
    CU_ASSERT(d_is(&INT_ENTRY, &local_data.list) == TRUE);
    CU_ASSERT(d_is(&STRING_ENTRY, &local_data.list) == TRUE);

    TEST("set an unset value and check ...");
    d_set(&UNSET_ENTRY, &local_data.list);
    CU_ASSERT(d_is(&SET_ENTRY, &local_data.list) == TRUE);
    CU_ASSERT(d_is(&UNSET_ENTRY, &local_data.list) == FALSE);
    CU_ASSERT(d_is(&INT_ENTRY, &local_data.list) == TRUE);
    CU_ASSERT(d_is(&STRING_ENTRY, &local_data.list) == TRUE);


    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_set(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);
    TEST("set an set and check it.");
    d_set(&SET_ENTRY, &local_data.list);
    CU_ASSERT(d_is(&SET_ENTRY, &local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("dubble set an set and check it.");
    d_set(&SET_ENTRY, &local_data.list);
    d_set(&SET_ENTRY, &local_data.list);
    d_set(&SET_ENTRY, &local_data.list);
    CU_ASSERT(d_is(&SET_ENTRY, &local_data.list) == TRUE);
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);

    TEST("set some fail types, and check");
    d_set(&INT_ENTRY, &local_data.list);
    d_set(&STRING_ENTRY, &local_data.list);
    CU_ASSERT(d_is(&INT_ENTRY, &local_data.list) == FALSE);
    CU_ASSERT(d_is(&STRING_ENTRY, &local_data.list) == FALSE);


    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_remove(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);
    TEST("Fill and string and int, remove the string, and make sure the int is still there.");
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("filling"));
    d_set_int(&INT_ENTRY, &local_data.list, 99);
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 99);
    d_remove(&STRING_ENTRY, &local_data.list);
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 99);
    d_remove_all(&local_data.list);
    CU_ASSERT(list_empty(&local_data.list) == TRUE);
    INIT_LIST_HEAD(&local_data.list);
    /*d_remove(s_entry * type, struct list_head * d); */
}

static void test_d_remove_all(void)
{
    NEW_TEST();
    INIT_LIST_HEAD(&local_data.list);
    TEST("Filling the list, and calling d_remove_all, then check that is it empty.");

    d_set_string(&STRING_ENTRY, &local_data.list, strdup("filling"));
    d_set_int(&INT_ENTRY, &local_data.list, 23);
    d_set_another_string(&STRINGS_ENTRY, &local_data.list, strdup("another"));
    d_set_another_string(&STRINGS_ENTRY, &local_data.list, strdup("string"));
    d_set(&SET_ENTRY, &local_data.list);
    CU_ASSERT(list_empty(&local_data.list) == FALSE);
    d_remove_all(&local_data.list);
    CU_ASSERT(list_empty(&local_data.list) == TRUE);

    INIT_LIST_HEAD(&local_data.list);
}

static void test_d_copy_all(void)
{
    NEW_TEST();
    static s_data tmp;

    INIT_LIST_HEAD(&local_data.list);
    INIT_LIST_HEAD(&tmp.list);

    TEST("Set some options, and get them again..\n");

    d_set_int(&INT_ENTRY, &local_data.list, 23);
    d_set_string(&STRING_ENTRY, &local_data.list, strdup("test-string"));
    CU_ASSERT(d_get_int(&INT_ENTRY, &local_data.list) == 23);
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &local_data.list),
                           "test-string");

    TEST("Copy all to tmp..\n");
    d_copy_all(&local_data.list, &tmp.list);
    CU_ASSERT(d_get_int(&INT_ENTRY, &tmp.list) == 23);
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &tmp.list),
                           "test-string");

    TEST("Remove all in global, and look on copy..");
    d_remove_all(&local_data.list);
    INIT_LIST_HEAD(&local_data.list);
    CU_ASSERT(d_get_int(&INT_ENTRY, &tmp.list) == 23);
    CU_ASSERT_STRING_EQUAL(d_get_string(&STRING_ENTRY, &tmp.list),
                           "test-string");

    INIT_LIST_HEAD(&tmp.list);

}

void test_initng_struct_data_setup(void)
{
    CU_pSuite suite;

    suite = CU_add_suite("initng_struct_data", empty_init, empty_cleanup);

    CU_add_test(suite, "d_get_string", test_d_get_string);
    CU_add_test(suite, "d_get_int", test_d_get_int);
    CU_add_test(suite, "d_get_next_string", test_d_get_next_string);
    CU_add_test(suite, "d_set_another_string", test_d_set_another_string);
    CU_add_test(suite, "d_set_string", test_d_set_string);
    CU_add_test(suite, "d_set_int", test_d_set_int);
    CU_add_test(suite, "d_count_type", test_d_count_type);
    CU_add_test(suite, "d_is", test_d_is);
    CU_add_test(suite, "d_set", test_d_set);
    CU_add_test(suite, "d_remove", test_d_remove);
    CU_add_test(suite, "d_remove_all", test_d_remove_all);
    CU_add_test(suite, "d_copy_all", test_d_copy_all);

}
